// $Id: NodesView.java,v 1.22 2000/11/21 00:17:52 kmbarber Exp $


class NodesView
    extends javax.swing.JFrame
{
    javax.swing.JPanel panel;
    javax.swing.JTable table;
    javax.swing.JPanel innerPanel;
    javax.swing.JScrollPane sp;
    java.util.Vector vector;
    java.util.Vector nodeViewVector;
    final Viewer viewer;
    javax.swing.JFrame frame;
    javax.swing.JFrame infoFrame;
    javax.swing.JFrame sourceFrame;
    String uri;
    int statementCount;
    int nodeCount;
    
    /**
     * display a list of all Resources
     */
    NodesView(Viewer viewer)
    {
	super("DAML Viewer - Nodes: " + viewer.uri);
	panel = new javax.swing.JPanel();
	panel.setLayout(new java.awt.BorderLayout());
	innerPanel = new javax.swing.JPanel();
	innerPanel.setLayout(new java.awt.GridLayout(0,1));
	this.viewer = viewer;
	statementCount = viewer.nstatements;
	uri = viewer.uri;
	frame = this;
	nodeViewVector = new java.util.Vector();

	// add sorted Nodes to table
	java.util.Vector nodes = new java.util.Vector();
	vector = new java.util.Vector();
	java.util.Iterator iterator = viewer.resources.values().iterator();
	// create an int to help in counting nodes
	nodeCount = 0;
	while (iterator.hasNext())
	    {
		Resource resource = (Resource) iterator.next();

		// skip Statements
		if (resource.value.startsWith("urn:rdf:SHA"))
		    continue;
		nodeCount++;
		nodes.add(resource);
	    }
	java.util.Collections.sort(nodes);
	iterator = nodes.iterator();
	while (iterator.hasNext())
	    {
		Resource resource = (Resource) iterator.next();
		java.util.Vector v = new java.util.Vector();
		v.add(resource);
		vector.add(v);
	    }
	
	// column headings
	java.util.Vector colNames = new java.util.Vector();
	colNames.add("Resources");
	
	table = new javax.swing.JTable(vector, colNames);

	table.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
	javax.swing.ListSelectionModel rowSM = table.getSelectionModel();
	rowSM.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
	    public void valueChanged(javax.swing.event.ListSelectionEvent e)
		{
		    //Ignore extra messages.
		    if (e.getValueIsAdjusting()) return;
		    
		    javax.swing.ListSelectionModel lsm = (javax.swing.ListSelectionModel) e.getSource();
		    if (lsm.isSelectionEmpty())
			{
			    //no rows are selected
			}
		    else
			{
			    int selectedRow = lsm.getMinSelectionIndex();
			    
			    int row = table.getSelectedRow();
			    int col = table.getSelectedColumn();
			    Node node = (Node) ((java.util.Vector) vector.elementAt(row)).elementAt(col);
			    if ((node != null)
				&& (node instanceof Resource))
				{
				   NodeView nv =  new NodeView(node);
				   nodeViewVector.add(nv);
				}
			}
		}
	});
	
	// create a menubar for frame
	javax.swing.JMenuBar mb = new javax.swing.JMenuBar();
	
	javax.swing.JMenu fileMenu = new javax.swing.JMenu("File");
	fileMenu.setMnemonic('F');
	javax.swing.JMenu helpMenu = new javax.swing.JMenu("Help");
	helpMenu.setMnemonic('H');

	javax.swing.JMenuItem inputURI = new javax.swing.JMenuItem("Open URI");
	inputURI.setMnemonic('I');
	javax.swing.JMenuItem info = new javax.swing.JMenuItem("Properties");
	info.setMnemonic('n');
	javax.swing.JMenuItem source = new javax.swing.JMenuItem("View Source");
	source.setMnemonic('V');
	javax.swing.JMenuItem closeWindows = new javax.swing.JMenuItem("Close all node view windows");
	closeWindows.setMnemonic('C');
	javax.swing.JMenuItem exit = new javax.swing.JMenuItem("Exit DAML Viewer");
	exit.setMnemonic('E');
	if(viewer.isApplet())
	    exit.setEnabled(false);
	javax.swing.JMenuItem about = new javax.swing.JMenuItem("About DAML Viewer");
	about.setMnemonic('A');
	javax.swing.JMenuItem help = new javax.swing.JMenuItem("Help");
	help.setMnemonic('H');
	help.setEnabled(false); // XXX
	
	// add action listeners to menubar items
	inputURI.addActionListener(new java.awt.event.ActionListener()
				   {
				       public void actionPerformed(java.awt.event.ActionEvent e)
					   {
					       // open URI Input window
					       new URIInput();
					   }
				   });
	info.addActionListener(new java.awt.event.ActionListener()
			       {
				   public void actionPerformed(java.awt.event.ActionEvent e)
				       {
					   // pop up a properties window that contains
					   // Current URI
					   // node count
					   // statment count
					   infoFrame = new javax.swing.JFrame("DAML Viewer Properties");
					   javax.swing.JPanel panel = new javax.swing.JPanel();
					   panel.setLayout(new java.awt.GridBagLayout());
					   java.awt.GridBagConstraints gbc = new java.awt.GridBagConstraints();
					   gbc.insets = new java.awt.Insets(5,5,5,5);
					   javax.swing.JLabel uriLabel = 
					       new javax.swing.JLabel("The current URI is: " + uri);
					   javax.swing.JLabel numNodes = 
					       new javax.swing.JLabel("The number of nodes is: " + nodeCount);
					   javax.swing.JLabel numStatements = 
					       new javax.swing.JLabel("The number of statements is: " + statementCount);
					   javax.swing.JButton ok = new javax.swing.JButton("OK");
					   ok.addActionListener(new java.awt.event.ActionListener(){
					       public void actionPerformed(java.awt.event.ActionEvent e)
						   {
						       infoFrame.dispose();
						   }
					   });
					   gbc.gridx = 0;
					   gbc.gridy = 0;
					   panel.add(uriLabel, gbc);
					   gbc.gridy = 1;
					   panel.add(numNodes, gbc);
					   gbc.gridy = 2;
					   panel.add(numStatements, gbc);
					   gbc.gridy = 3;
					   gbc.weighty = 1;
					   gbc.fill = java.awt.GridBagConstraints.NONE;
					   gbc.anchor = java.awt.GridBagConstraints.SOUTH;
					   panel.add(ok, gbc);
					   infoFrame.getContentPane().add(panel);
					   int width = 500;
					   int height = 150;
					   java.awt.Dimension screen = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
					   int x = (screen.width - width) / 2;
					   int y = (screen.height - height) / 2;
					   infoFrame.setBounds(x, y, width, height);
					   infoFrame.setVisible(true);
				       }
			       });
	source.addActionListener(new java.awt.event.ActionListener()
				 {
				     public void actionPerformed(java.awt.event.ActionEvent e)
					 {
					     // pop up a text area window and put the source in it
					     sourceFrame = new javax.swing.JFrame("DAML Viewer - Source:  " + NodesView.this.viewer.uri);
					     javax.swing.JPanel panel = new javax.swing.JPanel();
					     javax.swing.JScrollPane sp;
					     panel.setLayout(new java.awt.GridBagLayout());
					     java.awt.GridBagConstraints gbc = new java.awt.GridBagConstraints();
					     gbc.insets = new java.awt.Insets(5,5,5,5);
					     javax.swing.JTextArea jta = new javax.swing.JTextArea();
					     jta.setLineWrap(true);
					     jta.setWrapStyleWord(true);
					     // get source code. If as a string (??) use .append(String)
					     // If as a file ... need to add it when its created I think. XXX
					     gbc.gridy = 0;
					     gbc.gridx = 0;
					     gbc.fill = java.awt.GridBagConstraints.BOTH;
					     gbc.weightx = 1;
					     gbc.weighty = 1;
					     sp = new javax.swing.JScrollPane(jta);
					     sp.setVerticalScrollBarPolicy
						 (javax.swing.JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
					     panel.add(sp, gbc);
					     sourceFrame.getContentPane().add(panel);
					     sourceFrame.setDefaultCloseOperation(javax.swing.JFrame.DISPOSE_ON_CLOSE);
					     int width = 700;
					     int height = 700;
					     java.awt.Dimension sc = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
					     int x = (sc.width - width) / 2;
					     int y = (sc.height - height) / 2;
					     sourceFrame.setBounds(x, y, width, height);
					     // get source
					     try {
						 java.net.URL url = new java.net.URL(NodesView.this.viewer.uri);
						 java.io.BufferedReader stream = new java.io.BufferedReader(new java.io.InputStreamReader(url.openStream()));
						 String line;
						 while ((line = stream.readLine()) != null)
						     jta.append(line + "\n");
					     } catch (Exception ex) {
						 javax.swing.JOptionPane.showMessageDialog(frame,
											   ex,
											   "DAML Viewer Source",
											   javax.swing.JOptionPane.ERROR_MESSAGE);
						 return;
					     }
					     sourceFrame.setVisible(true);
					 }
				 });
	closeWindows.addActionListener(new java.awt.event.ActionListener()
				       {
					   public void actionPerformed(java.awt.event.ActionEvent e)
					       {
						   // close all nodeView windows, leaving nodesView window open 
						   java.util.Iterator it = nodeViewVector.iterator();
						   while(it.hasNext())
						       {
							   NodeView nv = (NodeView) it.next();
							   closeNodeWindow(nv);
						       }
						   
					       }
				       });
	exit.addActionListener(new java.awt.event.ActionListener()
			       {
				   public void actionPerformed(java.awt.event.ActionEvent e)
				       {
					   int retVal = javax.swing.JOptionPane.showConfirmDialog
					       (frame, 
						"Do you want to terminate the application and close all windows?",
						"Close Confirmation", 
						javax.swing.JOptionPane.YES_NO_CANCEL_OPTION);
					   if(retVal == javax.swing.JOptionPane.YES_OPTION)
					       {
						   // do a system.exit(0);
						   frame.dispose();
						   Viewer.exit(0);
					       }
					   else if(retVal == javax.swing.JOptionPane.NO_OPTION)
					       {
						   // call close then dispose()
						   java.util.Iterator it = nodeViewVector.iterator();
						   while(it.hasNext())
						       {
							   NodeView nv = (NodeView) it.next();
							   closeNodeWindow(nv);
						       }
						   frame.dispose();
					       }
					   else
					       {
						   // do nothing
					       }
				       }
			       });
	
	help.addActionListener(new java.awt.event.ActionListener()
			       {
				   public void actionPerformed(java.awt.event.ActionEvent e)
				       {
					   // pop up a help window XXX
				       }
			       });
	about.addActionListener(new java.awt.event.ActionListener()
			       {
				   public void actionPerformed(java.awt.event.ActionEvent e)
				       {
					   // pop up an about window
					   javax.swing.JOptionPane.showMessageDialog
					       (frame, 
						Viewer.getVersionString(),
						"About DAML Viewer", 
						javax.swing.JOptionPane.INFORMATION_MESSAGE);
				       }
			       });

	// add menuItems to menus, and menus to menubar
	fileMenu.add(inputURI);
	fileMenu.add(info);
	fileMenu.add(source);
	fileMenu.add(closeWindows);
	fileMenu.add(exit);
	helpMenu.add(help);
	helpMenu.addSeparator();
	helpMenu.add(about);

	mb.add(fileMenu);
	mb.add(helpMenu);
	this.setJMenuBar(mb);
	
	// test to see if its an applet ..
	if(viewer.isApplet())
	    // do nothing
	    this.setDefaultCloseOperation(javax.swing.JFrame.DO_NOTHING_ON_CLOSE);
	else
	    {
		// call closeNodeWindow stuff then dispose();
		this.addWindowListener(new java.awt.event.WindowAdapter()
				       {
					   public void windowClosing(java.awt.event.WindowEvent e)
					       {
						  java.util.Iterator it = nodeViewVector.iterator();
						  while(it.hasNext())
						      {
							  NodeView nv = (NodeView) it.next();
							  closeNodeWindow(nv);
						      }
						  frame.dispose();
					       }
				       });
	    }

	sp = new javax.swing.JScrollPane(table);
	panel.add(sp, java.awt.BorderLayout.CENTER);
	this.getContentPane().add(panel);
	int width = 800;
	int height = 400;
	java.awt.Dimension screen = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
	int x = (screen.width - width) / 2;
	int y = (screen.height - height) / 2;
	this.setBounds(x, y, width, height);
	this.setVisible(true);
    }

    void closeNodeWindow(NodeView nv)
    {
	java.util.Iterator i = nv.nvVector.iterator();
	while(i.hasNext())
	    {
		NodeView view = (NodeView) i.next();
		closeNodeWindow(view);
	    }
	try { 
	    nv.dispose();
	} catch (Exception e) {
	    e.printStackTrace();
	    javax.swing.JOptionPane.showMessageDialog(frame, 
						      e, 
						      "closing NodeView window", 
						      javax.swing.JOptionPane.ERROR_MESSAGE);
	}
    }
    
}





